#pragma once

#include <string>
#include <vector>

#include "FieldData.h"
#include "IDArray.h"

namespace VectorDB {

/**
 * @brief Topk results for one target vector of MilvusClient::Search()
 */
struct SingleResult {
    /**
     * @brief Constructor
     */
    SingleResult(IDArray&& ids, std::vector<float>&& scores, std::vector<FieldDataPtr>&& output_fields);

    /**
     * @brief Distances/scores array of one target vector
     */
    const std::vector<float>& Scores() const;

    /**
     * @brief Topk id array of one target vector
     */
    const IDArray& Ids() const;

    /**
     * @brief Output fields data
     */
    const std::vector<FieldDataPtr>& OutputFields() const;

    /**
     * @brief Get an output field by name
     */
    FieldDataPtr OutputField(const std::string& name) const;

private:
    IDArray ids_;
    std::vector<float> scores_;
    std::vector<FieldDataPtr> output_fields_;
};

/**
 * @brief Results returned by MilvusClient::Search().
 */
class SearchResults {
public:
    SearchResults();

    /**
     * @brief Constructor
     */
    explicit SearchResults(std::vector<SingleResult>&& results);

    /**
     * @brief Get search results.
     */
    std::vector<SingleResult>& Results();

private:
    std::vector<SingleResult> nq_results_{};
};

}  // namespace VectorDB
