#pragma once

#include <string>

/**
 * @brief Milvus SDK namespace
 */
namespace VectorDB {

/**
 * @brief Status code for SDK interface return
 */
enum class StatusCode {
    OK = 0,

    // system error section
    UNKNOWN_ERROR = 1,
    NOT_SUPPORTED,
    NOT_CONNECTED,

    // function error section
    INVALID_AGUMENT = 1000,
    RPC_FAILED,
    SERVER_FAILED,
    TIMEOUT,

    // validation error
    DIMENSION_NOT_EQUAL = 2000,
    VECTOR_IS_EMPTY,
    JSON_PARSE_ERROR,
};

/**
 * @brief Status code and message returned by SDK interface.
 */
class Status {
public:
    /**
     * @brief Constructor of Status
     */
    Status(StatusCode code, std::string msg);
    Status();

    /**
     * @brief A success status
     */
    static Status OK();

    /**
     * @brief Indicate the status is ok
     */
    bool IsOk() const;

    /**
     * @brief Return the status code
     */
    StatusCode Code() const;

    /**
     * @brief Return the error message
     */
    const std::string& Message() const;

private:
    StatusCode code_{StatusCode::OK};
    std::string msg_{"OK"};
};  // Status

}  // namespace VectorDB
